from django.core.management.base import BaseCommand
from django.contrib.auth.models import User, Group

from userextensions.models import ServiceAccount


class Command(BaseCommand):
    help = 'Create service account'

    def __init__(self):
        self.opts = None
        super(Command, self).__init__()

    def add_arguments(self, parser):
        """ define command arguments """
        parser.add_argument('--username', type=str, help='name for new superuser')
        parser.add_argument('--group', type=str, help='group this service account will be a member of')
        parser.add_argument('--description', type=str, help='description of this service account')

    def handle(self, *args, **options):
        """ command entry point """
        self.opts = options

        if not self.opts['username']:
            print('Please provide a username.')
            return

        if not self.opts['group']:
            print('Please specify a group')
            return

        # first check if user exists
        if User.objects.filter(username=self.opts['username']):
            print(f"A users with the username {self.opts['username']} already exists; please specify a "
                  f"different username")
            return

        try:
            user, is_user_new = User.objects.get_or_create(username=self.opts['username'])
            group, is_group_new = Group.objects.get_or_create(name=self.opts['group'])
            group.user_set.add(user)
            srv_acct = ServiceAccount.objects.create(user=user, group=group, description=self.opts['description'])
            print(f'Successfully created service account {srv_acct}')
        except Exception as err:
            print(f'Failed to create service account {err} in group {group}')



"""
Views used specifically for handling AJAX Requests
"""
# import django modules
from django.template import loader
from handyhelpers.views.ajax import AjaxGetView

# import models
from django.contrib.auth.models import Group
from userextensions.models import ServiceAccountTokenHistory


class GetUsersPerGroup(AjaxGetView):
    """
    Description:
        Get all users in a group.
    Args:
        request: AJAX request object.
    Returns:
        HttpResponse: JSON formatted response.
    """
    template = loader.get_template('userextensions/ajax/get_users_per_group.htm')
    
    def get(self, request, *args, **kwargs):
        obj_id = request.GET['client_response']
        self.data = Group.objects.get(id=obj_id).user_set.all().order_by('username')
        return super().get(request, *args, **kwargs)


class GetSerivceAccountTokenHistory(AjaxGetView):
    """
    Description:
        Get the API token refresh history for a give ServiceAccount.
    Args:
        request: AJAX request object.
    Returns:
        HttpResponse: JSON formatted response.
    """
    template = loader.get_template('userextensions/ajax/get_token_history_for_srv_acct.htm')
    
    def get(self, request, *args, **kwargs):
        obj_id = request.GET['client_response']
        self.data = ServiceAccountTokenHistory.objects.filter(srv_acct__id=obj_id).order_by('-timestamp')
        return super().get(request, *args, **kwargs)


class GetSerivceAccountToken(AjaxGetView):
    """
    Description:
        Show the API token for a service account.
    Args:
        request: AJAX request object.
    Returns:
        HttpResponse: JSON formatted response.
    """
    template = loader.get_template('userextensions/ajax/show_srv_acct_token.htm')
    
    def get(self, request, *args, **kwargs):
        self.data = request.GET['client_response']
        return super().get(request, *args, **kwargs)



# Generated by Django 4.2 on 2024-03-06 22:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("userextensions", "0007_auto_20220807_2104"),
    ]

    operations = [
        migrations.AlterField(
            model_name="serviceaccount",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="serviceaccounttokenhistory",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="theme",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="userfavorite",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="userpreference",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="userpreference",
            name="timezone",
            field=models.CharField(
                blank=True,
                default="UTC",
                help_text="timezone to use when displaying time and datetime values",
                max_length=32,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="userrecent",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
    ]



# Generated by Django 3.2.13 on 2022-08-07 21:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('userextensions', '0006_auto_20210507_1831'),
    ]

    operations = [
        migrations.AddField(
            model_name='userpreference',
            name='help_text_enable',
            field=models.BooleanField(default=True, help_text='enable help text and tooltips'),
        ),
        migrations.AddField(
            model_name='userpreference',
            name='timezone',
            field=models.CharField(blank=True, help_text='timezone to use when displaying time and datetime values', max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name='serviceaccount',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='serviceaccounttokenhistory',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='theme',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='userfavorite',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='userpreference',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='userrecent',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]
